from go_libs.dll_path_finder import get_dll_path

if get_dll_path():
    from go_libs.go_libs import get_twodhist_file
    from go_libs.go_libs import get_twodhist
    from go_libs.go_libs import get_twodhist_parallel
    from go_libs.go_libs import get_imhist
